@@ -1,4 +1,3 @@
-extern crate gtk;
 use crate::gui_data::GuiData;
 use gtk::prelude::*;
 
@@ -9,22 +8,25 @@ const INSTRUCTION_SITE: &str = "https://github.com/qarmin/czkawka/blob/master/in
 pub fn connect_about_buttons(gui_data: &GuiData) {
     let button_donation = gui_data.about.button_donation.clone();
     button_donation.connect_clicked(move |_| {
-        if open::that(SPONSOR_SITE).is_err() {
-            println!("Failed to open sponsor site: {}", SPONSOR_SITE)
-        };
+        open::that_in_background(SPONSOR_SITE);
+        // if let Err(e) = open::that(SPONSOR_SITE) {
+        //     println!("Failed to open sponsor site: {}, reason {}", SPONSOR_SITE, e)
+        // };
     });
 
     let button_instruction = gui_data.about.button_instruction.clone();
     button_instruction.connect_clicked(move |_| {
-        if open::that(INSTRUCTION_SITE).is_err() {
-            println!("Failed to open instruction site: {}", INSTRUCTION_SITE)
-        };
+        open::that_in_background(INSTRUCTION_SITE);
+        // if let Err(e) = open::that(INSTRUCTION_SITE) {
+        //     println!("Failed to open instruction site: {}, reason {}", INSTRUCTION_SITE, e)
+        // };
     });
 
     let button_repository = gui_data.about.button_repository.clone();
     button_repository.connect_clicked(move |_| {
-        if open::that(REPOSITORY_SITE).is_err() {
-            println!("Failed to open repository site: {}", REPOSITORY_SITE)
-        };
+        open::that_in_background(REPOSITORY_SITE);
+        // if let Err(e) = open::that(REPOSITORY_SITE) {
+        //     println!("Failed to open repository site: {}, reason {}", REPOSITORY_SITE, e)
+        // };
     });
 }
