@@ -1,6 +1,5 @@
 use czkawka_core::*;
 
-extern crate gtk;
 use crate::gui_data::GuiData;
 use crate::help_functions::*;
 use crate::notebook_enums::*;
@@ -17,6 +16,7 @@ use czkawka_core::zeroed::ZeroedFiles;
 use glib::Sender;
 use gtk::prelude::*;
 use gtk::WindowPosition;
+use img_hash::{FilterType, HashAlg};
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Arc;
 use std::thread;
@@ -47,22 +47,20 @@ pub fn connect_button_search(
     let check_button_recursive = gui_data.upper_notebook.check_button_recursive.clone();
     let entry_excluded_items = gui_data.upper_notebook.entry_excluded_items.clone();
     let entry_same_music_minimal_size = gui_data.main_notebook.entry_same_music_minimal_size.clone();
+    let entry_same_music_maximal_size = gui_data.main_notebook.entry_same_music_maximal_size.clone();
     let entry_allowed_extensions = gui_data.upper_notebook.entry_allowed_extensions.clone();
     let buttons_names = gui_data.bottom_buttons.buttons_names.clone();
     let radio_button_duplicates_name = gui_data.main_notebook.radio_button_duplicates_name.clone();
     let radio_button_duplicates_size = gui_data.main_notebook.radio_button_duplicates_size.clone();
     let radio_button_duplicates_hashmb = gui_data.main_notebook.radio_button_duplicates_hashmb.clone();
     let radio_button_duplicates_hash = gui_data.main_notebook.radio_button_duplicates_hash.clone();
-    let radio_button_similar_images_minimal = gui_data.main_notebook.radio_button_similar_images_minimal.clone();
-    let radio_button_similar_images_very_small = gui_data.main_notebook.radio_button_similar_images_very_small.clone();
-    let radio_button_similar_images_small = gui_data.main_notebook.radio_button_similar_images_small.clone();
-    let radio_button_similar_images_medium = gui_data.main_notebook.radio_button_similar_images_medium.clone();
-    let radio_button_similar_images_high = gui_data.main_notebook.radio_button_similar_images_high.clone();
-    let radio_button_similar_images_very_high = gui_data.main_notebook.radio_button_similar_images_very_high.clone();
+    let scale_similarity = gui_data.main_notebook.scale_similarity.clone();
     let entry_duplicate_minimal_size = gui_data.main_notebook.entry_duplicate_minimal_size.clone();
+    let entry_duplicate_maximal_size = gui_data.main_notebook.entry_duplicate_maximal_size.clone();
     let stop_receiver = gui_data.stop_receiver.clone();
     let entry_big_files_number = gui_data.main_notebook.entry_big_files_number.clone();
     let entry_similar_images_minimal_size = gui_data.main_notebook.entry_similar_images_minimal_size.clone();
+    let entry_similar_images_maximal_size = gui_data.main_notebook.entry_similar_images_maximal_size.clone();
     let check_button_music_title: gtk::CheckButton = gui_data.main_notebook.check_button_music_title.clone();
     let check_button_music_artist: gtk::CheckButton = gui_data.main_notebook.check_button_music_artist.clone();
     let check_button_music_album_title: gtk::CheckButton = gui_data.main_notebook.check_button_music_album_title.clone();
@@ -93,16 +91,29 @@ pub fn connect_button_search(
     let check_button_settings_hide_hard_links = gui_data.settings.check_button_settings_hide_hard_links.clone();
     let check_button_settings_use_cache = gui_data.settings.check_button_settings_use_cache.clone();
     let entry_settings_cache_file_minimal_size = gui_data.settings.entry_settings_cache_file_minimal_size.clone();
+    let radio_button_similar_hash_size_4 = gui_data.main_notebook.radio_button_similar_hash_size_4.clone();
+    let radio_button_similar_hash_size_8 = gui_data.main_notebook.radio_button_similar_hash_size_8.clone();
+    let radio_button_similar_hash_size_16 = gui_data.main_notebook.radio_button_similar_hash_size_16.clone();
+    let radio_button_resize_algorithm_catmullrom = gui_data.main_notebook.radio_button_resize_algorithm_catmullrom.clone();
+    let radio_button_resize_algorithm_lanczos3 = gui_data.main_notebook.radio_button_resize_algorithm_lanczos3.clone();
+    let radio_button_resize_algorithm_nearest = gui_data.main_notebook.radio_button_resize_algorithm_nearest.clone();
+    let radio_button_resize_algorithm_triangle = gui_data.main_notebook.radio_button_resize_algorithm_triangle.clone();
+    let radio_button_resize_algorithm_gaussian = gui_data.main_notebook.radio_button_resize_algorithm_gaussian.clone();
+    let radio_button_similar_hash_algorithm_gradient = gui_data.main_notebook.radio_button_similar_hash_algorithm_gradient.clone();
+    let radio_button_similar_hash_algorithm_blockhash = gui_data.main_notebook.radio_button_similar_hash_algorithm_blockhash.clone();
+    let radio_button_similar_hash_algorithm_mean = gui_data.main_notebook.radio_button_similar_hash_algorithm_mean.clone();
+    let radio_button_similar_hash_algorithm_vertgradient = gui_data.main_notebook.radio_button_similar_hash_algorithm_vertgradient.clone();
+    let radio_button_similar_hash_algorithm_doublegradient = gui_data.main_notebook.radio_button_similar_hash_algorithm_doublegradient.clone();
 
     buttons_search_clone.connect_clicked(move |_| {
         let included_directories = get_path_buf_from_vector_of_strings(get_string_from_list_store(&tree_view_included_directories));
         let excluded_directories = get_path_buf_from_vector_of_strings(get_string_from_list_store(&tree_view_excluded_directories));
-        let recursive_search = check_button_recursive.get_active();
-        let excluded_items = entry_excluded_items.get_text().as_str().to_string().split(',').map(|e| e.to_string()).collect::<Vec<String>>();
-        let allowed_extensions = entry_allowed_extensions.get_text().as_str().to_string();
-        let hide_hard_links = check_button_settings_hide_hard_links.get_active();
-        let use_cache = check_button_settings_use_cache.get_active();
-        let minimal_cache_file_size = entry_settings_cache_file_minimal_size.get_text().as_str().parse::<u64>().unwrap_or(2 * 1024 * 1024);
+        let recursive_search = check_button_recursive.is_active();
+        let excluded_items = entry_excluded_items.text().as_str().to_string().split(',').map(|e| e.to_string()).collect::<Vec<String>>();
+        let allowed_extensions = entry_allowed_extensions.text().as_str().to_string();
+        let hide_hard_links = check_button_settings_hide_hard_links.is_active();
+        let use_cache = check_button_settings_use_cache.is_active();
+        let minimal_cache_file_size = entry_settings_cache_file_minimal_size.text().as_str().parse::<u64>().unwrap_or(2 * 1024 * 1024);
 
         let show_dialog = Arc::new(AtomicBool::new(true));
 
@@ -114,7 +125,7 @@ pub fn connect_button_search(
         entry_info.set_text("Searching data, it may take a while, please wait...");
 
         // Set dialog to center to current screen(it is impossible to center it to main window)
-        window_progress.set_position(WindowPosition::CenterAlways);
+        window_progress.set_position(WindowPosition::Center);
 
         // Resets progress bars
         progress_bar_all_stages.set_fraction(0 as f64);
@@ -125,7 +136,7 @@ pub fn connect_button_search(
         let glib_stop_sender = glib_stop_sender.clone();
         let stop_receiver = stop_receiver.clone();
 
-        match to_notebook_main_enum(notebook_main.get_current_page().unwrap()) {
+        match to_notebook_main_enum(notebook_main.current_page().unwrap()) {
             NotebookMainEnum::Duplicate => {
                 label_stage.show();
                 grid_progress_stages.show_all();
@@ -134,25 +145,26 @@ pub fn connect_button_search(
                 get_list_store(&tree_view_duplicate_finder).clear();
 
                 let check_method;
-                if radio_button_duplicates_name.get_active() {
+                if radio_button_duplicates_name.is_active() {
                     check_method = duplicate::CheckingMethod::Name;
-                } else if radio_button_duplicates_size.get_active() {
+                } else if radio_button_duplicates_size.is_active() {
                     check_method = duplicate::CheckingMethod::Size;
-                } else if radio_button_duplicates_hashmb.get_active() {
+                } else if radio_button_duplicates_hashmb.is_active() {
                     check_method = duplicate::CheckingMethod::HashMb;
-                } else if radio_button_duplicates_hash.get_active() {
+                } else if radio_button_duplicates_hash.is_active() {
                     check_method = duplicate::CheckingMethod::Hash;
                 } else {
                     panic!("No radio button is pressed");
                 }
-                let minimal_file_size = entry_duplicate_minimal_size.get_text().as_str().parse::<u64>().unwrap_or(1024);
+                let minimal_file_size = entry_duplicate_minimal_size.text().as_str().parse::<u64>().unwrap_or(1024 * 8);
+                let maximal_file_size = entry_duplicate_maximal_size.text().as_str().parse::<u64>().unwrap_or(1024 * 1024 * 1024 * 1024);
 
                 let hash_type: HashType;
-                if radio_button_hash_type_blake3.get_active() {
+                if radio_button_hash_type_blake3.is_active() {
                     hash_type = duplicate::HashType::Blake3;
-                } else if radio_button_hash_type_crc32.get_active() {
+                } else if radio_button_hash_type_crc32.is_active() {
                     hash_type = duplicate::HashType::Crc32;
-                } else if radio_button_hash_type_xxh3.get_active() {
+                } else if radio_button_hash_type_xxh3.is_active() {
                     hash_type = duplicate::HashType::Xxh3;
                 } else {
                     panic!("No radio button is pressed");
@@ -168,6 +180,7 @@ pub fn connect_button_search(
                     df.set_excluded_items(excluded_items);
                     df.set_allowed_extensions(allowed_extensions);
                     df.set_minimal_file_size(minimal_file_size);
+                    df.set_maximal_file_size(maximal_file_size);
                     df.set_minimal_cache_file_size(minimal_cache_file_size);
                     df.set_check_method(check_method);
                     df.set_hash_type(hash_type);
@@ -223,7 +236,7 @@ pub fn connect_button_search(
 
                 get_list_store(&tree_view_big_files_finder).clear();
 
-                let numbers_of_files_to_check = entry_big_files_number.get_text().as_str().parse::<usize>().unwrap_or(50);
+                let numbers_of_files_to_check = entry_big_files_number.text().as_str().parse::<usize>().unwrap_or(50);
 
                 let futures_sender_big_file = futures_sender_big_file.clone();
                 // Find big files
@@ -268,25 +281,52 @@ pub fn connect_button_search(
 
                 get_list_store(&tree_view_similar_images_finder).clear();
 
-                let minimal_file_size = entry_similar_images_minimal_size.get_text().as_str().parse::<u64>().unwrap_or(1024 * 16);
-
-                let similarity;
-                if radio_button_similar_images_minimal.get_active() {
-                    similarity = similar_images::Similarity::Minimal;
-                } else if radio_button_similar_images_very_small.get_active() {
-                    similarity = similar_images::Similarity::VerySmall;
-                } else if radio_button_similar_images_small.get_active() {
-                    similarity = similar_images::Similarity::Small;
-                } else if radio_button_similar_images_medium.get_active() {
-                    similarity = similar_images::Similarity::Medium;
-                } else if radio_button_similar_images_high.get_active() {
-                    similarity = similar_images::Similarity::High;
-                } else if radio_button_similar_images_very_high.get_active() {
-                    similarity = similar_images::Similarity::VeryHigh;
+                let hash_size;
+                if radio_button_similar_hash_size_4.is_active() {
+                    hash_size = 4;
+                } else if radio_button_similar_hash_size_8.is_active() {
+                    hash_size = 8;
+                } else if radio_button_similar_hash_size_16.is_active() {
+                    hash_size = 16;
                 } else {
                     panic!("No radio button is pressed");
                 }
 
+                let image_filter;
+                if radio_button_resize_algorithm_catmullrom.is_active() {
+                    image_filter = FilterType::CatmullRom;
+                } else if radio_button_resize_algorithm_lanczos3.is_active() {
+                    image_filter = FilterType::Lanczos3;
+                } else if radio_button_resize_algorithm_nearest.is_active() {
+                    image_filter = FilterType::Nearest;
+                } else if radio_button_resize_algorithm_triangle.is_active() {
+                    image_filter = FilterType::Triangle;
+                } else if radio_button_resize_algorithm_gaussian.is_active() {
+                    image_filter = FilterType::Gaussian;
+                } else {
+                    panic!("No radio button is pressed");
+                }
+
+                let hash_alg;
+                if radio_button_similar_hash_algorithm_blockhash.is_active() {
+                    hash_alg = HashAlg::Blockhash;
+                } else if radio_button_similar_hash_algorithm_gradient.is_active() {
+                    hash_alg = HashAlg::Gradient;
+                } else if radio_button_similar_hash_algorithm_mean.is_active() {
+                    hash_alg = HashAlg::Mean;
+                } else if radio_button_similar_hash_algorithm_vertgradient.is_active() {
+                    hash_alg = HashAlg::VertGradient;
+                } else if radio_button_similar_hash_algorithm_doublegradient.is_active() {
+                    hash_alg = HashAlg::DoubleGradient;
+                } else {
+                    panic!("No radio button is pressed");
+                }
+
+                let minimal_file_size = entry_similar_images_minimal_size.text().as_str().parse::<u64>().unwrap_or(1024 * 16);
+                let maximal_file_size = entry_similar_images_maximal_size.text().as_str().parse::<u64>().unwrap_or(1024 * 1024 * 1024 * 1024);
+
+                let similarity = similar_images::Similarity::Similar(scale_similarity.value() as u32);
+
                 let futures_sender_similar_images = futures_sender_similar_images.clone();
                 // Find similar images
                 thread::spawn(move || {
@@ -297,8 +337,12 @@ pub fn connect_button_search(
                     sf.set_recursive_search(recursive_search);
                     sf.set_excluded_items(excluded_items);
                     sf.set_minimal_file_size(minimal_file_size);
+                    sf.set_maximal_file_size(maximal_file_size);
                     sf.set_similarity(similarity);
                     sf.set_use_cache(use_cache);
+                    sf.set_hash_alg(hash_alg);
+                    sf.set_hash_size(hash_size);
+                    sf.set_image_filter(image_filter);
                     sf.find_similar_images(Some(&stop_receiver), Some(&futures_sender_similar_images));
                     let _ = glib_stop_sender.send(Message::SimilarImages(sf));
                 });
@@ -331,23 +375,24 @@ pub fn connect_button_search(
 
                 get_list_store(&tree_view_same_music_finder).clear();
 
-                let minimal_file_size = entry_same_music_minimal_size.get_text().as_str().parse::<u64>().unwrap_or(1024);
+                let minimal_file_size = entry_same_music_minimal_size.text().as_str().parse::<u64>().unwrap_or(1024 * 8);
+                let maximal_file_size = entry_same_music_maximal_size.text().as_str().parse::<u64>().unwrap_or(1024 * 1024 * 1024 * 1024);
 
                 let mut music_similarity: MusicSimilarity = MusicSimilarity::NONE;
 
-                if check_button_music_title.get_active() {
+                if check_button_music_title.is_active() {
                     music_similarity |= MusicSimilarity::TITLE;
                 }
-                if check_button_music_artist.get_active() {
+                if check_button_music_artist.is_active() {
                     music_similarity |= MusicSimilarity::ARTIST;
                 }
-                if check_button_music_album_title.get_active() {
+                if check_button_music_album_title.is_active() {
                     music_similarity |= MusicSimilarity::ALBUM_TITLE;
                 }
-                if check_button_music_album_artist.get_active() {
+                if check_button_music_album_artist.is_active() {
                     music_similarity |= MusicSimilarity::ALBUM_ARTIST;
                 }
-                if check_button_music_year.get_active() {
+                if check_button_music_year.is_active() {
                     music_similarity |= MusicSimilarity::YEAR;
                 }
 
@@ -361,6 +406,7 @@ pub fn connect_button_search(
                         mf.set_excluded_directory(excluded_directories);
                         mf.set_excluded_items(excluded_items);
                         mf.set_minimal_file_size(minimal_file_size);
+                        mf.set_maximal_file_size(maximal_file_size);
                         mf.set_recursive_search(recursive_search);
                         mf.set_music_similarity(music_similarity);
                         mf.find_same_music(Some(&stop_receiver), Some(&futures_sender_same_music));
